package com.tzld.piaoquan.longarticle.service.local.impl;

import com.tzld.piaoquan.longarticle.common.enums.ContentStatusEnum;
import com.tzld.piaoquan.longarticle.common.enums.SourceTypesEnum;
import com.tzld.piaoquan.longarticle.component.RedisLock;
import com.tzld.piaoquan.longarticle.dao.mapper.LongArticlesTextMapper;
import com.tzld.piaoquan.longarticle.dao.mapper.MatchVideoMapper;
import com.tzld.piaoquan.longarticle.model.bo.MatchContent;
import com.tzld.piaoquan.longarticle.model.dto.MiniprogramCardRequest;
import com.tzld.piaoquan.longarticle.model.po.LongArticlesText;
import com.tzld.piaoquan.longarticle.model.po.LongArticlesTextExample;
import com.tzld.piaoquan.longarticle.model.po.MatchVideo;
import com.tzld.piaoquan.longarticle.model.po.MatchVideoExample;
import com.tzld.piaoquan.longarticle.model.vo.MatchMiniprogramStatusParam;
import com.tzld.piaoquan.longarticle.service.local.KimiService;
import com.tzld.piaoquan.longarticle.utils.page.Page;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.redis.core.RedisTemplate;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.util.CollectionUtils;

import java.util.ArrayList;
import java.util.List;
import java.util.Objects;
import java.util.UUID;
import java.util.concurrent.*;

@Slf4j
@Service
public class MatchVideoServiceImpl {


    private static final String KIMI_LOCK_KEY = "kimi_lock_key_%s";

    private static final String CRAWLER_LOCK_KEY = "crawler_lock_key_%s";


    @Autowired
    KimiService kimiService;

    @Autowired
    private MatchVideoMapper matchVideoMapper;

    @Autowired
    private LongArticlesTextMapper longArticlesTextMapper;

    @Autowired
    private CrawlerVideoServiceImpl crawlerVideoService;

    @Autowired
    private RedisLock redisLock;

    @Autowired
    private RedisTemplate<String, Object> redisTemplate;


    // 定义一个阻塞队列
    private static final ArrayBlockingQueue<MatchVideo> matchVideoQueue = new ArrayBlockingQueue<>(100000);


    private static final int size = 5;
    // 定义一个线程池，设置消费线程的数量
    private static final ThreadPoolExecutor matchVideoPoolExecutor = (ThreadPoolExecutor) Executors.newFixedThreadPool(size);

    @Transactional
    public void addMatchVideo(MatchContent matchContent) {
        MatchVideoExample example = new MatchVideoExample();
        example.createCriteria().andContentIdEqualTo(matchContent.getSourceId()).andGhIdEqualTo(matchContent.getGhId());
        long l = matchVideoMapper.countByExample(example);
        if (l > 0) {
            return;
        }
        String traceId = "search-" + UUID.randomUUID();
        MatchVideo matchVideo = new MatchVideo();
        matchVideo.setTraceId(traceId);
        matchVideo.setContentId(matchContent.getSourceId());
        matchVideo.setAccountName(matchContent.getAccountName());
        matchVideo.setGhId(matchContent.getGhId());
        matchVideo.setFlowPoolLevel(matchContent.getFlowPoolLevelTag());
        matchVideo.setContentStatus(0);
        matchVideo.setPublishFlag(2);
        long timestamp = System.currentTimeMillis() / 1000;
        matchVideo.setContentStatusUpdateTime(Long.valueOf(timestamp).intValue());
        matchVideo.setRequestTimestamp(Long.valueOf(timestamp).intValue());
        matchVideoMapper.insertSelective(matchVideo);
        LongArticlesTextExample longArticlesTextExample = new LongArticlesTextExample();
        longArticlesTextExample.createCriteria().andContentIdEqualTo(matchContent.getSourceId());
        long l1 = longArticlesTextMapper.countByExample(longArticlesTextExample);
        if (l1 == 0) {
            LongArticlesText longArticlesText = new LongArticlesText();
            longArticlesText.setArticleTitle(matchContent.getTitle());
            longArticlesText.setArticleText(matchContent.getContent());
            longArticlesText.setContentId(matchContent.getSourceId());
            longArticlesText.setKimiStatus(0);
            longArticlesTextMapper.insertSelective(longArticlesText);
        }
    }

    public void getMatchVideo() {
        if (matchVideoPoolExecutor.getCorePoolSize() - matchVideoPoolExecutor.getActiveCount() > 0) {
            int threadSize = matchVideoPoolExecutor.getCorePoolSize() - matchVideoPoolExecutor.getActiveCount();
            log.info("threadNum={}", threadSize);
            CountDownLatch countDownLatch = new CountDownLatch(threadSize);
            // 启动消费者线程
            for (int i = 0; i < threadSize; i++) {
                matchVideoPoolExecutor.submit(new Thread(() -> {
                    log.info("启动匹配小程序线程");
                    while (true) {
                        try {
                            // 超过 5 分钟没有数据，销毁当前线程
                            MatchVideo matchVideo = matchVideoQueue.poll(5, TimeUnit.MINUTES); // 等待最多 5 分钟
                            if (matchVideo == null) {
                                break; // 退出当前线程
                            }
                            processMatchContent(matchVideo);
                        } catch (InterruptedException e) {
                            throw new RuntimeException(e);
                        }
                    }
                    log.info("启动匹配小程序线程结束");
                    countDownLatch.countDown();
                }));
            }
        }


        List<Integer> targetStatus = new ArrayList<Integer>() {{
            add(0);
            add(1);
            add(2);
        }};
        List<MatchVideo> matchVideos;
        //循环增加游标位置，通过主键索引过滤已经处理的数据
        do {
            Integer id = (Integer) redisTemplate.opsForValue().get("last_match_video_id");
            if (id == null) {
                id = 0;
            }
            MatchVideoExample example = new MatchVideoExample();
            example.createCriteria().andIdGreaterThan(id);
            example.setOrderByClause("id asc");
            Page page = new Page<>();
            page.setCurrentPage(1);
            page.setPageSize(5000);
            example.setPage(page);
            matchVideos = matchVideoMapper.selectByExample(example);
            boolean flag = true;
            for (MatchVideo matchVideo : matchVideos) {
                if (targetStatus.contains(matchVideo.getContentStatus())) {
                    flag = false;
                    break;
                }
            }
            if (flag) {
                Integer lastId = matchVideos.get(matchVideos.size() - 1).getId();
                redisTemplate.opsForValue().set("last_match_video_id", lastId);
            }
        } while (CollectionUtils.isEmpty(matchVideos));

        int pageNum = 1;
        do {
            Integer id = (Integer) redisTemplate.opsForValue().get("last_match_video_id");
            if (id == null) {
                break;
            }

            int pageSize = 5000;
            MatchVideoExample example = new MatchVideoExample();
            example.createCriteria().andIdGreaterThan(id).andContentStatusIn(targetStatus);
            example.setOrderByClause("id asc");
            Page page = new Page<>();
            page.setCurrentPage(pageNum);
            page.setPageSize(pageSize);
            example.setPage(page);
            matchVideos = matchVideoMapper.selectByExample(example);
            matchVideoQueue.addAll(matchVideos);
            pageNum++;
        } while (CollectionUtils.isEmpty(matchVideos));

    }


    public void processMatchContent(MatchVideo matchVideo) {
        if (matchVideo.getContentStatus() == 0) {
            //1.执行kimi任务
            LongArticlesText kimiText = kimiService.getKimiText(matchVideo.getContentId());
            if (kimiText == null) {
                //TODO 查询信息重新生成kimi信息
                return;
            }
            //执行kimi任务
            if (kimiText.getKimiStatus() == 0) {
                String lockKey = String.format(KIMI_LOCK_KEY, matchVideo.getContentId());
                String lockValue = UUID.randomUUID().toString();
                boolean lock = redisLock.tryLock(lockKey, lockValue, 300, TimeUnit.SECONDS);
                if (lock) {
                    kimiText = kimiService.getAndUpdateContent(matchVideo.getContentId());
                    redisLock.unlock(lockKey, lockValue);
                    if (kimiText == null) {
                        return;
                    }
                } else {
                    return;
                }
            } else {
                //更新状态为kimi执行完成
                updateStatus(matchVideo.getId(), 1);
            }
        }
        boolean existCrawlerVideo = crawlerVideoService.existCrawlerVideo(matchVideo.getContentId());
        if (!existCrawlerVideo) {
            String lockKey = String.format(CRAWLER_LOCK_KEY, matchVideo.getContentId());
            String lockValue = UUID.randomUUID().toString();
            boolean lock = redisLock.tryLock(lockKey, lockValue, 600, TimeUnit.SECONDS);
            if (lock) {
                LongArticlesText kimiText = kimiService.getKimiText(matchVideo.getContentId());
                crawlerVideoService.addCrawlerVideo(matchVideo.getContentId(), kimiText);
                redisLock.unlock(lockKey, lockValue);
            }
        } else {
            //更新状态为etl执行完成
            updateStatus(matchVideo.getId(), 3);
        }
    }

    private void updateStatus(Integer id, Integer status) {
        MatchVideo matchVideo = new MatchVideo();
        matchVideo.setId(id);
        matchVideo.setContentStatus(status);
        matchVideoMapper.updateByPrimaryKeySelective(matchVideo);
    }


}



